##
# Copyright 2020-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Metagenome-Atlas, implemented as an easyblock.

@author: Pavel Grochal (INUITS)
"""
import os
import stat

from easybuild.tools.filetools import adjust_permissions, write_file
from easybuild.easyblocks.generic.pythonpackage import PythonPackage


class EB_Metagenome_Atlas(PythonPackage):
    """
    Support for building/installing Metagenome-Atlas.
    """

    def post_install_step(self):
        """Create snakemake config files"""

        # https://metagenome-atlas.readthedocs.io/en/latest/usage/getting_started.html#set-up-of-cluster-execution
        # Files were obtained by:
        # cookiecutter https://github.com/metagenome-atlas/clusterprofile.git -o ~/.config/snakemake

        snakemake_conf_dir = os.path.join(self.installdir, 'snakemake-config')

        cluster_config_file = os.path.join(snakemake_conf_dir, 'cluster_config.yaml')
        config_file = os.path.join(snakemake_conf_dir, 'config.yaml')
        key_mapping = os.path.join(snakemake_conf_dir, 'key_mapping.yaml')
        lsf_status = os.path.join(snakemake_conf_dir, 'lsf_status.py')
        pbs_status = os.path.join(snakemake_conf_dir, 'pbs_status.py')
        scheduler = os.path.join(snakemake_conf_dir, 'scheduler.py')
        slurm_status = os.path.join(snakemake_conf_dir, 'slurm_status.py')

        write_file(cluster_config_file, CLUSTER_CONFIG_FILE_TEXT)
        write_file(config_file, CONFIG_FILE_TEXT % cluster_config_file)
        write_file(key_mapping, KEY_MAPPING_TEXT)
        write_file(lsf_status, LSF_STATUS_TEXT)
        write_file(pbs_status, PBS_STATUS_TEXT)
        # scheduler file includes small tweak from original file in git
        write_file(scheduler, SCHEDULER_TEXT)
        write_file(slurm_status, SLURM_STATUS_TEXT)

        # make sure .py scripts are executable
        adjust_permissions(lsf_status, stat.S_IXUSR)
        adjust_permissions(pbs_status, stat.S_IXUSR)
        adjust_permissions(scheduler, stat.S_IXUSR)
        adjust_permissions(slurm_status, stat.S_IXUSR)

    def sanity_check_step(self):
        """
        Custom sanity check for Metagenome-Atlas.
        """
        custom_paths = {
            'files': ['bin/atlas'],
            'dirs': [os.path.join('lib', 'python%(pyshortver)s', 'site-packages')],
        }
        custom_commands = [
            'atlas --version',
            'atlas init --help',
            'atlas run --help',
        ]
        super(PythonPackage, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)


# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/cluster_config.yaml
CLUSTER_CONFIG_FILE_TEXT = r"""
__default__:
  nodes: 1
"""

# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/config.yaml
# modified cores to 1
CONFIG_FILE_TEXT = r"""
restart-times: 0
cluster-config: "%s" #abs path
cluster: "scheduler.py" #
cluster-status: "slurm_status.py" #
max-jobs-per-second: 10
max-status-checks-per-second: 10
cores: 1 # how many jobs you want to submit to your cluster queue
local-cores: 1
rerun-incomplete: true  # recomended for cluster submissions
keep-going: false
"""

# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/key_mapping.yaml
KEY_MAPPING_TEXT = r"""
# only parameters defined in key_mapping (see below) are passed to the command in the order specified.
system: "slurm" #check if system is defined below

slurm:
  command: "sbatch --parsable"
  key_mapping:
    name: "--job-name={}"
    threads: "-n {}"
    mem: "--mem={}g"
    account: "--account={}"
    queue: "--partition={}"
    time: "--time={}"
    nodes: "-N {}"
pbs:
  command: "qsub"
  key_mapping:
    name: "-N {}"
    account: "-A {}"
    queue: "-q {}"
    threads: "-l nodes=1:ppn={}" # always use 1 node
    mem: "-l mem={}gb"
    time: "-l walltime={}00" #min= seconds x 100
lsf:
  command: "bsub -e lsf_%J.log -o lsf_%J.log"
  key_mapping:
    queue: "-q {}"
    name: "-J {}"
    threads: "-n {}"
    mem: '-R "rusage[mem={}000]"'
    account: "-P {}"
    nodes: "-C {}"

# for other cluster systems see: https://slurm.schedmd.com/rosetta.pdf
"""

# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/lsf_status.py
LSF_STATUS_TEXT = r"""
#!/usr/bin/env python3

import os
import sys
import warnings
import subprocess

jobid = sys.argv[1]

out= subprocess.run(['bjobs','-noheader',jobid],stdout=subprocess.PIPE).stdout.decode('utf-8')

state = out.strip().split()[2]

map_state={"PEND":'running',
           "RUN":'running',
           "PROV":"running",
           "WAIT":'running',
           "DONE":'success',
           "":'success'}

print(map_state.get(state,'failed'))
"""

# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/pbs_status.py
PBS_STATUS_TEXT = r"""
#!/usr/bin/env python3

import sys
import subprocess
import xml.etree.cElementTree as ET

jobid = sys.argv[1]

try:
    res = subprocess.run("qstat -f -x {}".format(jobid), check=True,
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)

    xmldoc = ET.ElementTree(ET.fromstring(res.stdout.decode())).getroot()
    job_state = xmldoc.findall('.//job_state')[0].text

    if job_state == "C":
        exit_status = xmldoc.findall('.//exit_status')[0].text
        if exit_status == '0':
            print("success")
        else:
            print("failed")
    else:
        print("running")

except (subprocess.CalledProcessError, IndexError, KeyboardInterrupt) as e:
    print("failed")
"""

# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/scheduler.py
# enhanced with parsing below comment '# get jobid if the string contains cluster name as well separated by ;'
SCHEDULER_TEXT = r"""
#!/usr/bin/env python3

import sys, os
from subprocess import Popen, PIPE
import yaml

def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)

# let snakemake read job_properties
from snakemake.utils import read_job_properties

jobscript = sys.argv[1]
job_properties = read_job_properties(jobscript)

#default paramters defined in cluster_spec (accessed via snakemake read_job_properties)
cluster_param= job_properties["cluster"]

if job_properties["type"]=='single':
    cluster_param['name'] = job_properties['rule']
elif job_properties["type"]=='group':
    cluster_param['name'] = job_properties['groupid']
else:
    raise NotImplementedError(f"Don't know what to do with job_properties['type']=={job_properties['type']}")

# don't overwrite default parameters if defined in rule (or config file)
if ('threads' in job_properties) and ('threads' not in cluster_param):
    cluster_param["threads"] = job_properties["threads"]
for res in ['time','mem']:
    if (res in job_properties["resources"]) and (res not in cluster_param):
        cluster_param[res] = job_properties["resources"][res]

# time in hours
if "time" in cluster_param:
    cluster_param["time"]=int(cluster_param["time"]*60)

# check which system you are on and load command command_options
key_mapping_file=os.path.join(os.path.dirname(__file__),"key_mapping.yaml")
command_options=yaml.load(open(key_mapping_file),
                          Loader=yaml.BaseLoader)
system= command_options['system']
command= command_options[system]['command']

key_mapping= command_options[system]['key_mapping']

# construct command:
for  key in key_mapping:
    if key in cluster_param:
        command+=" "
        command+=key_mapping[key].format(cluster_param[key])

command+=' {}'.format(jobscript)

eprint("submit command: "+command)

p = Popen(command.split(' '), stdout=PIPE, stderr=PIPE)
output, error = p.communicate()
if p.returncode != 0:
    raise Exception("Job can't be submitted\n"+output.decode("utf-8")+error.decode("utf-8"))
else:
    res= output.decode("utf-8")

    if system=='lsf':
        import re
        match = re.search(r"Job <(\d+)> is submitted", res)
        jobid = match.group(1)

    elif system=='pbs':
        jobid= res.strip().split('.')[0]

    else:
        try:
            jobid= int(res.strip().split()[-1])
        except ValueError as e:
            # get jobid if the string contains cluster name as well separated by ;
            jobid = int(res.strip().split()[-1].split(';')[0])

    print(jobid)
"""

# obtained from
# https://github.com/metagenome-atlas/clusterprofile/blob/master/{{cookiecutter.profile_name}}/slurm_status.py
SLURM_STATUS_TEXT = r"""
#!/usr/bin/env python3

import re
import subprocess as sp
import shlex
import sys
import time
import logging
logger = logging.getLogger("__name__")

STATUS_ATTEMPTS = 20

jobid = sys.argv[1]

for i in range(STATUS_ATTEMPTS):
    try:
        sacct_res = sp.check_output(shlex.split("sacct -P -b -j {} -n".format(jobid)))
        res = {x.split("|")[0]: x.split("|")[1] for x in sacct_res.decode().strip().split("\n")}
        break
    except sp.CalledProcessError as e:
        logger.error("sacct process error")
        logger.error(e)
    except IndexError as e:
        pass
    # Try getting job with scontrol instead in case sacct is misconfigured
    try:
        sctrl_res = sp.check_output(shlex.split("scontrol -o show job {}".format(jobid)))
        m = re.search("JobState=(\w+)", sctrl_res.decode())
        res = {jobid: m.group(1)}
        break
    except sp.CalledProcessError as e:
        logger.error("scontrol process error")
        logger.error(e)
        if i >= STATUS_ATTEMPTS - 1:
            print("failed")
            exit(0)
        else:
            time.sleep(1)

status = res[jobid]

if (status == "BOOT_FAIL"):
    print("failed")
elif (status == "OUT_OF_MEMORY"):
    print("failed")
elif (status.startswith("CANCELLED")):
    print("failed")
elif (status == "COMPLETED"):
    print("success")
elif (status == "DEADLINE"):
    print("failed")
elif (status == "FAILED"):
    print("failed")
elif (status == "NODE_FAIL"):
    print("failed")
elif (status == "PREEMPTED"):
    print("failed")
elif (status == "TIMEOUT"):
    print("failed")
# Unclear whether SUSPENDED should be treated as running or failed
elif (status == "SUSPENDED"):
    print("failed")
else:
    print("running")
"""



##
# Copyright 2021-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing torchvision, implemented as an easyblock

@author: Alexander Grund (TU Dresden)
@author: Kenneth Hoste (HPC-UGent)
"""
from easybuild.easyblocks.generic.pythonpackage import PythonPackage, det_pylibdir
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.config import build_option
from easybuild.tools.modules import get_software_version
import easybuild.tools.environment as env


class EB_torchvision(PythonPackage):
    """Support for building/installing TorchVison."""

    @staticmethod
    def extra_options():
        """Change some defaults for easyconfig parameters."""
        extra_vars = PythonPackage.extra_options()
        extra_vars['use_pip'][0] = True
        extra_vars['download_dep_fail'][0] = True
        extra_vars['sanity_pip_check'][0] = True
        return extra_vars

    def __init__(self, *args, **kwargs):
        """Initialize torchvision easyblock."""
        super(EB_torchvision, self).__init__(*args, **kwargs)

        dep_names = set(dep['name'] for dep in self.cfg.dependencies())

        # require that PyTorch is listed as dependency
        if 'PyTorch' not in dep_names:
            raise EasyBuildError('PyTorch not found as a dependency')

        # enable building with GPU support if CUDA is included as dependency
        if 'CUDA' in dep_names:
            self.with_cuda = True
        else:
            self.with_cuda = False

    def configure_step(self):
        """Set up torchvision config"""

        # Note: Those can be overwritten by e.g. preinstallopts
        env.setvar('BUILD_VERSION', self.version)
        env.setvar('PYTORCH_VERSION', get_software_version('PyTorch'))

        if self.with_cuda:
            # make sure that torchvision is installed with CUDA support by setting $FORCE_CUDA
            env.setvar('FORCE_CUDA', '1')
            # specify CUDA compute capabilities via $TORCH_CUDA_ARCH_LIST
            cuda_cc = self.cfg['cuda_compute_capabilities'] or build_option('cuda_compute_capabilities')
            if cuda_cc:
                env.setvar('TORCH_CUDA_ARCH_LIST', ';'.join(cuda_cc))

        super(EB_torchvision, self).configure_step()

    def sanity_check_step(self):
        """Custom sanity check for torchvision."""
        custom_commands = None
        custom_paths = None

        # check whether torchvision was indeed built with CUDA support,
        # cfr. https://discuss.pytorch.org/t/notimplementederror-could-not-run-torchvision-nms-with-arguments-from-\
        #      the-cuda-backend-this-could-be-because-the-operator-doesnt-exist-for-this-backend/132352/4
        if self.with_cuda:
            custom_commands = []
            python_code = '; '.join([
                "import torch, torchvision",
                "boxes = torch.tensor([[0., 1., 2., 3.]]).to('cuda')",
                "scores = torch.randn(1).to('cuda')",
                "print(torchvision.ops.nms(boxes, scores, 0.5))",
            ])
            custom_commands.append('python -c "%s"' % python_code)
            custom_paths = {
                'files': [],
                'dirs': [det_pylibdir()],
            }

        return super(EB_torchvision, self).sanity_check_step(custom_commands=custom_commands, custom_paths=custom_paths)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Bazel, implemented as an easyblock
"""
from easybuild.tools import LooseVersion
import glob
import os
import tempfile

import easybuild.tools.environment as env
from easybuild.framework.easyblock import EasyBlock
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.filetools import apply_regex_substitutions, copy_file, which
from easybuild.tools.modules import get_software_root, get_software_version
from easybuild.tools.run import run_cmd
from easybuild.framework.easyconfig import CUSTOM


class EB_Bazel(EasyBlock):
    """Support for building/installing Bazel."""

    @staticmethod
    def extra_options():
        """Extra easyconfig parameters specific to EB_Bazel."""
        extra_vars = {
            'static': [None, 'Build statically linked executables ' +
                             '(default: True for Bazel >= 1.0 else False)', CUSTOM],
        }
        return EasyBlock.extra_options(extra_vars)

    def fixup_hardcoded_paths(self):
        """Patch out hard coded paths to /tmp, compiler and binutils tools"""
        # replace hardcoded /tmp in java build scripts
        regex_subs = [
            (r'`mktemp -d /tmp/tmp.XXXXXXXXXX`', '$$(mktemp -d $${TMPDIR:-/tmp}/tmp.XXXXXXXXXX)'),
        ]
        filepath = os.path.join('src', 'main', 'java', 'com', 'google', 'devtools', 'build', 'lib', 'BUILD')
        if os.path.exists(filepath):
            apply_regex_substitutions(filepath, regex_subs)

        binutils_root = get_software_root('binutils')
        gcc_root = get_software_root('GCCcore') or get_software_root('GCC')
        gcc_ver = get_software_version('GCCcore') or get_software_version('GCC')

        # only patch Bazel scripts if binutils & GCC installation prefix could be determined
        if not binutils_root or not gcc_root:
            self.log.info("Not patching Bazel build scripts, installation prefix for binutils/GCC not found")
            return

        # replace hardcoded paths in (unix_)cc_configure.bzl
        # hard-coded paths in (unix_)cc_configure.bzl were removed in 0.19.0
        if LooseVersion(self.version) < LooseVersion('0.19.0'):
            regex_subs = [
                (r'-B/usr/bin', '-B%s' % os.path.join(binutils_root, 'bin')),
                (r'"/usr/bin', '"' + os.path.join(binutils_root, 'bin')),
            ]
            for conf_bzl in ['cc_configure.bzl', 'unix_cc_configure.bzl']:
                filepath = os.path.join('tools', 'cpp', conf_bzl)
                if os.path.exists(filepath):
                    apply_regex_substitutions(filepath, regex_subs)

        # replace hardcoded paths in CROSSTOOL
        # CROSSTOOL script is no longer there in Bazel 0.24.0
        if LooseVersion(self.version) < LooseVersion('0.24.0'):
            res = glob.glob(os.path.join(gcc_root, 'lib', 'gcc', '*', gcc_ver, 'include'))
            if res and len(res) == 1:
                gcc_lib_inc = res[0]
            else:
                raise EasyBuildError("Failed to pinpoint location of GCC include files: %s", res)

            gcc_lib_inc_bis = os.path.join(os.path.dirname(gcc_lib_inc), 'include-fixed')
            if not os.path.exists(gcc_lib_inc_bis):
                self.log.info("Derived directory %s does not exist, falling back to %s", gcc_lib_inc_bis, gcc_lib_inc)
                gcc_lib_inc_bis = gcc_lib_inc

            gcc_cplusplus_inc = os.path.join(gcc_root, 'include', 'c++', gcc_ver)
            if not os.path.exists(gcc_cplusplus_inc):
                raise EasyBuildError("Derived directory %s does not exist", gcc_cplusplus_inc)

            regex_subs = [
                (r'-B/usr/bin', '-B%s' % os.path.join(binutils_root, 'bin')),
                (r'(cxx_builtin_include_directory:.*)/usr/lib/gcc', r'\1%s' % gcc_lib_inc),
                (r'(cxx_builtin_include_directory:.*)/usr/local/include', r'\1%s' % gcc_lib_inc_bis),
                (r'(cxx_builtin_include_directory:.*)/usr/include', r'\1%s' % gcc_cplusplus_inc),
            ]
            for tool in ['ar', 'cpp', 'dwp', 'gcc', 'ld']:
                path = which(tool)
                if path:
                    regex_subs.append((os.path.join('/usr', 'bin', tool), path))
                else:
                    raise EasyBuildError("Failed to determine path to '%s'", tool)

            apply_regex_substitutions(os.path.join('tools', 'cpp', 'CROSSTOOL'), regex_subs)

    def prepare_step(self, *args, **kwargs):
        """Setup bazel output root"""
        super(EB_Bazel, self).prepare_step(*args, **kwargs)
        self.bazel_tmp_dir = tempfile.mkdtemp(suffix='-bazel-tmp', dir=self.builddir)
        self._make_output_user_root()

    def _make_output_user_root(self):
        if not os.path.isdir(self.builddir):
            # Can happen on module-only or sanity-check-only runs
            self.log.info("Using temporary folder for user_root as builddir doesn't exist")
            dir = None  # Will use the EB created temp dir
        else:
            dir = self.builddir
        self._output_user_root = tempfile.mkdtemp(suffix='-bazel-root', dir=dir)

    @property
    def output_user_root(self):
        try:
            return self._output_user_root
        except AttributeError:
            self._make_output_user_root()
            return self._output_user_root

    def extract_step(self):
        """Extract Bazel sources."""
        # Older Bazel won't build when the output_user_root is a subfolder of the source folder
        # So create a dedicated source folder
        self.cfg.update('unpack_options', '-d src')
        super(EB_Bazel, self).extract_step()

    def configure_step(self):
        """Custom configuration procedure for Bazel."""

        # Last instance of hardcoded compiler/binutils paths was removed in 0.24.0, however
        # hardcoded /tmp affects all versions
        self.fixup_hardcoded_paths()

        # Keep temporary directory in case of error. EB will clean it up on success
        apply_regex_substitutions(os.path.join('scripts', 'bootstrap', 'buildenv.sh'), [
            (r'atexit cleanup_tempdir_.*', '')
        ])

        # enable building in parallel
        bazel_args = '--jobs=%d' % self.cfg['parallel']

        # Bazel provides a JDK by itself for some architectures
        # We want to enforce it using the JDK we provided via modules
        # This is required for Power where Bazel does not have a JDK, but requires it for building itself
        # See https://github.com/bazelbuild/bazel/issues/10377
        bazel_args += ' --host_javabase=@local_jdk//:jdk'

        # Link C++ libs statically, see https://github.com/bazelbuild/bazel/issues/4137
        static = self.cfg['static']
        if static is None:
            # Works for Bazel 1.x and higher
            static = LooseVersion(self.version) >= LooseVersion('1.0.0')
        if static:
            env.setvar('BAZEL_LINKOPTS', '-static-libstdc++:-static-libgcc')
            env.setvar('BAZEL_LINKLIBS', '-l%:libstdc++.a')

        env.setvar('EXTRA_BAZEL_ARGS', bazel_args)
        env.setvar('EMBED_LABEL', self.version)
        env.setvar('VERBOSE', 'yes')

    def build_step(self):
        """Custom build procedure for Bazel."""
        cmd = ' '.join([
            "export TMPDIR='%s' &&" % self.bazel_tmp_dir,  # The initial bootstrap of bazel is done in TMPDIR
            self.cfg['prebuildopts'],
            "bash -c 'set -x && ./compile.sh'",  # Show the commands the script is running to faster debug failures
        ])
        run_cmd(cmd, log_all=True, simple=True, log_ok=True)

    def test_step(self):
        """Test the compilation"""

        runtest = self.cfg['runtest']
        if runtest:
            # This could be used to pass options to Bazel: runtest = '--bazel-opt=foo test'
            if runtest is True:
                runtest = 'test'
            cmd = " ".join([
                self.cfg['pretestopts'],
                os.path.join('output', 'bazel'),
                # Avoid bazel using $HOME
                '--output_user_root=%s' % self.output_user_root,
                runtest,
                '--jobs=%d' % self.cfg['parallel'],
                '--host_javabase=@local_jdk//:jdk',
                # Be more verbose
                '--subcommands', '--verbose_failures',
                # Just build tests
                '--build_tests_only',
                self.cfg['testopts']
            ])
            run_cmd(cmd, log_all=True, simple=True)

    def install_step(self):
        """Custom install procedure for Bazel."""
        copy_file(os.path.join('output', 'bazel'), os.path.join(self.installdir, 'bin', 'bazel'))

    def sanity_check_step(self):
        """Custom sanity check for Bazel."""
        custom_paths = {
            'files': ['bin/bazel'],
            'dirs': [],
        }
        custom_commands = []
        if LooseVersion(self.version) >= LooseVersion('1.0'):
            # Avoid writes to $HOME
            custom_commands.append("bazel --output_user_root=%s --help" % self.output_user_root)

        super(EB_Bazel, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for Perl, implemented as an easyblock

@author: Jens Timmerman (Ghent University)
@author: Kenneth Hoste (Ghent University)
"""
from easybuild.tools import LooseVersion
import glob
import os
import stat

from easybuild.easyblocks.generic.configuremake import ConfigureMake
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.config import build_option
from easybuild.tools.filetools import adjust_permissions
from easybuild.tools.environment import setvar, unset_env_vars
from easybuild.tools.modules import get_software_root
from easybuild.tools.py2vs3 import string_type
from easybuild.tools.run import run_cmd

# perldoc -lm seems to be the safest way to test if a module is available, based on exit code
EXTS_FILTER_PERL_MODULES = ("perldoc -lm %(ext_name)s ", "")


class EB_Perl(ConfigureMake):
    """Support for building and installing Perl."""

    @staticmethod
    def extra_options():
        """Add extra config options specific to Perl."""
        extra_vars = {
            'use_perl_threads': [True, "Enable use of internal Perl threads via -Dusethreads configure option", CUSTOM],
        }
        return ConfigureMake.extra_options(extra_vars)

    def configure_step(self):
        """
        Configure Perl build: run ./Configure instead of ./configure with some different options
        """
        # avoid that $CPATH or $C_INCLUDE_PATH include an empty entry, since that makes Perl build fail miserably
        # see https://github.com/easybuilders/easybuild-easyconfigs/issues/8859
        for key in ['CPATH', 'C_INCLUDE_PATH']:
            value = os.getenv(key, None)
            if value is not None:
                paths = value.split(os.pathsep)
                if '' in paths:
                    self.log.info("Found empty entry in $%s, filtering it out...", key)
                    os.environ[key] = os.pathsep.join(p for p in paths if p)

        majver = self.version.split('.')[0]
        configopts = [
            self.cfg['configopts'],
            '-Dcc="{0}"'.format(os.getenv('CC')),
            '-Dccflags="{0}"'.format(os.getenv('CFLAGS')) if '-Dccflags' not in self.cfg['configopts'] else '',
            '-Dinc_version_list=none',
            '-Dprefix=%(installdir)s',
            # guarantee that scripts are installed in /bin in the installation directory (and not in a guessed path)
            # see https://github.com/easybuilders/easybuild-easyblocks/issues/1659
            '-Dinstallscript=%(installdir)s/bin',
            '-Dscriptdir=%(installdir)s/bin',
            '-Dscriptdirexp=%(installdir)s/bin',
            # guarantee that the install directory has the form lib/perlX/
            # see https://github.com/easybuilders/easybuild-easyblocks/issues/1700
            "-Dinstallstyle='lib/perl%s'" % majver,
        ]
        if self.cfg['use_perl_threads']:
            configopts.append('-Dusethreads')

        # see https://metacpan.org/pod/distribution/perl/INSTALL#Specifying-a-logical-root-directory
        sysroot = build_option('sysroot')
        if sysroot:
            configopts.append('-Dsysroot=%s' % sysroot)

            configopts.append('-Dlocincpth="%s"' % os.path.join(sysroot, 'usr', 'include'))

            # also specify 'lib*' subdirectories to consider in specified sysroot, via glibpth configure option;
            # we can list both lib64 and lib here, the Configure script will eliminate non-existing paths...
            sysroot_lib_paths = [
                os.path.join(sysroot, 'lib64'),
                os.path.join(sysroot, 'lib'),
                os.path.join(sysroot, 'usr', 'lib64'),
                os.path.join(sysroot, 'usr', 'lib'),
            ]
            configopts.append('-Dglibpth="%s"' % ' '.join(sysroot_lib_paths))

        configopts = (' '.join(configopts)) % {'installdir': self.installdir}

        # if $COLUMNS is set to 0, 'ls' produces a warning like:
        #   ls: ignoring invalid width in environment variable COLUMNS: 0
        # this confuses Perl's Configure script and makes it fail,
        # so just unset $COLUMNS if it set to 0...
        if os.getenv('COLUMNS', None) == '0':
            unset_env_vars(['COLUMNS'])

        cmd = '%s ./Configure -de %s' % (self.cfg['preconfigopts'], configopts)
        run_cmd(cmd, log_all=True, simple=True)

    def test_step(self):
        """Test Perl build via 'make test'."""
        # allow escaping with runtest = False
        if self.cfg['runtest'] is None or self.cfg['runtest']:
            parallel = self.cfg['parallel']
            if isinstance(self.cfg['runtest'], string_type):
                cmd = "make %s" % self.cfg['runtest']
            elif parallel and LooseVersion(self.version) >= LooseVersion('5.30.0'):
                # run tests in parallel, see https://perldoc.perl.org/perlhack#Parallel-tests;
                # only do this for Perl 5.30 and newer (conservative choice, actually supported in Perl >= 5.10.1)
                cmd = ' '.join([
                    'TEST_JOBS=%s' % parallel,
                    'PERL_TEST_HARNESS_ASAP=1',
                    "make -j %s test_harness" % parallel,
                ])
            else:
                cmd = "make test"

            # specify locale to be used, to avoid that a handful of tests fail
            cmd = "export LC_ALL=C && %s" % cmd

            run_cmd(cmd, log_all=False, log_ok=False, simple=False)

    def prepare_for_extensions(self):
        """
        Set default class and filter for Perl modules
        """
        # build and install additional modules with PerlModule easyblock
        self.cfg['exts_defaultclass'] = "PerlModule"
        self.cfg['exts_filter'] = EXTS_FILTER_PERL_MODULES

        sysroot = build_option('sysroot')
        if sysroot:
            # define $OPENSSL_PREFIX to ensure that Net-SSLeay extension picks up OpenSSL
            # from specified sysroot rather than from host OS
            setvar('OPENSSL_PREFIX', sysroot)

    def post_install_step(self, *args, **kwargs):
        """
        Custom post-installation step for Perl: avoid excessive long shebang lines in Perl scripts.
        """

        # if path to install directory is too long, we need to patch the shebang line in all Perl scripts;
        # there is a strict limit on the allowed shebang length (~128 characters)
        bin_path = os.path.join(self.installdir, 'bin')
        bin_perl = os.path.join(bin_path, 'perl')
        bin_perl_len = len(bin_perl)
        if bin_perl_len > 110:
            self.log.info("Path to 'perl' (%s) is too long (%d), we need to patch the shebang line in bin/*...",
                          bin_perl, bin_perl_len)

            # first make sure that files in bin/ are writable for current user
            bin_paths = glob.glob(os.path.join(bin_path, '*'))
            for bin_path in bin_paths:
                adjust_permissions(bin_path, stat.S_IWUSR, add=True, relative=True)

            # specify pattern for paths (relative to install dir) of files for which shebang should be patched
            self.cfg['fix_perl_shebang_for'] = 'bin/*'

        super(EB_Perl, self).post_install_step(*args, **kwargs)

    def sanity_check_step(self):
        """Custom sanity check for Perl."""
        majver = self.version.split('.')[0]
        dirs = ['lib/perl%s/%s' % (majver, self.version)]
        if get_software_root('groff'):
            dirs.extend(['man'])

        custom_paths = {
            'files': [os.path.join('bin', x) for x in ['perl', 'perldoc']],
            'dirs': dirs,
        }
        super(EB_Perl, self).sanity_check_step(custom_paths=custom_paths)


def get_major_perl_version():
    """"
    Returns the major verson of the perl binary in the current path
    """
    cmd = "perl -MConfig -e 'print $Config::Config{PERL_API_REVISION}'"
    (perlmajver, _) = run_cmd(cmd, log_all=True, log_output=True, simple=False)
    return perlmajver


def get_site_suffix(tag):
    """
    Returns the suffix for site* (e.g. sitearch, sitelib)
    this will look something like /lib/perl5/site_perl/5.16.3/x86_64-linux-thread-multi
    so, e.g. sitearch without site prefix

    @tag: site tag to use, e.g. 'sitearch', 'sitelib'
    """
    perl_cmd = 'my $a = $Config::Config{"%s"}; $a =~ s/($Config::Config{"siteprefix"})//; print $a' % tag
    cmd = "perl -MConfig -e '%s'" % perl_cmd
    (sitesuffix, _) = run_cmd(cmd, log_all=True, log_output=True, simple=False)
    # obtained value usually contains leading '/', so strip it off
    return sitesuffix.lstrip(os.path.sep)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for Trilinos, implemented as an easyblock

@author: Kenneth Hoste (Ghent University)
"""
import os
import random
import re

import easybuild.tools.toolchain as toolchain
from easybuild.easyblocks.generic.cmakemake import CMakeMake
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools import LooseVersion
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.config import build_path
from easybuild.tools.filetools import mkdir, remove_dir, symlink
from easybuild.tools.modules import get_software_root
from easybuild.tools.py2vs3 import ascii_letters
from easybuild.tools.systemtools import get_shared_lib_ext


class EB_Trilinos(CMakeMake):
    """Support for building Trilinos."""
    # see http://trilinos.sandia.gov/Trilinos10CMakeQuickstart.txt

    @staticmethod
    def extra_options():
        """Add extra config options specific to Trilinos."""
        extra_vars = {
            'shared_libs': [None, "Deprecated. Use build_shared_libs", CUSTOM],
            'openmp': [True, "Enable OpenMP support", CUSTOM],
            'forward_deps': [True, "Enable all forward dependencies", CUSTOM],
            'build_tests': [True, "Enable building tests/examples", CUSTOM],
            'all_exts': [True, "Enable all Trilinos packages", CUSTOM],
            'skip_exts': [[], "List of Trilinos packages to skip", CUSTOM],
            'verbose': [False, "Configure for verbose output", CUSTOM],
        }
        return CMakeMake.extra_options(extra_vars)

    def __init__(self, *args, **kwargs):
        """Constructor of custom easyblock for Trilinos."""
        super(EB_Trilinos, self).__init__(*args, **kwargs)

        if self.cfg['shared_libs'] is not None:
            self.log.deprecated("Use 'build_shared_libs' instead of 'shared_libs' easyconfig parameter", '5.0')
            self.cfg['build_shared_libs'] = self.cfg['shared_libs']

    def configure_step(self):
        """Set some extra environment variables before configuring."""

        # enable verbose output if desired
        if self.cfg['verbose']:
            for x in ["CONFIGURE", "MAKEFILE"]:
                self.cfg.update('configopts', "-DTrilinos_VERBOSE_%s:BOOL=ON" % x)

        # compiler flags
        cflags = [os.getenv('CFLAGS')]
        cxxflags = [os.getenv('CXXFLAGS')]
        fflags = [os.getenv('FFLAGS')]

        ignore_cxx_seek_mpis = [toolchain.INTELMPI, toolchain.MPICH,
                                toolchain.MPICH2, toolchain.MVAPICH2]  # @UndefinedVariable
        ignore_cxx_seek_flag = "-DMPICH_IGNORE_CXX_SEEK"
        if self.toolchain.mpi_family() in ignore_cxx_seek_mpis:
            cflags.append(ignore_cxx_seek_flag)
            cxxflags.append(ignore_cxx_seek_flag)
            fflags.append(ignore_cxx_seek_flag)

        self.cfg.update('configopts', '-DCMAKE_C_FLAGS="%s"' % ' '.join(cflags))
        self.cfg.update('configopts', '-DCMAKE_CXX_FLAGS="%s"' % ' '.join(cxxflags))
        self.cfg.update('configopts', '-DCMAKE_Fortran_FLAGS="%s"' % ' '.join(fflags))

        # Make sure Tpetra/Kokkos Serial mode is enabled regardless of OpenMP
        self.cfg.update('configopts', "-DKokkos_ENABLE_Serial:BOOL=ON")
        self.cfg.update('configopts', "-DTpetra_INST_SERIAL:BOOL=ON")

        # OpenMP
        if self.cfg['openmp']:
            self.cfg.update('configopts', "-DTrilinos_ENABLE_OpenMP:BOOL=ON")
            self.cfg.update('configopts', "-DKokkos_ENABLE_OpenMP:BOOL=ON")

        # MPI
        if self.toolchain.options.get('usempi', None):
            self.cfg.update('configopts', "-DTPL_ENABLE_MPI:BOOL=ON")

        if self.cfg['build_tests']:
            # enable full testing
            self.cfg.update('configopts', "-DTrilinos_ENABLE_TESTS:BOOL=ON")
        if self.cfg['forward_deps']:
            self.cfg.update('configopts', "-DTrilinos_ENABLE_ALL_FORWARD_DEP_PACKAGES:BOOL=ON")

        lib_re = re.compile("^lib(.*).a$")

        # BLAS, LAPACK
        for dep in ["BLAS", "LAPACK"]:
            self.cfg.update('configopts', '-DTPL_ENABLE_%s:BOOL=ON' % dep)
            libdirs = os.getenv('%s_LIB_DIR' % dep)
            if self.toolchain.comp_family() == toolchain.GCC:  # @UndefinedVariable
                libdirs += ";%s/lib64" % get_software_root('GCC')
            self.cfg.update('configopts', '-D%s_LIBRARY_DIRS="%s"' % (dep, libdirs))
            if self.cfg['openmp']:
                libs = os.getenv('%s_MT_STATIC_LIBS' % dep).split(',')
            else:
                libs = os.getenv('%s_STATIC_LIBS' % dep).split(',')
            lib_names = ';'.join([lib_re.search(x).group(1) for x in libs])
            if self.toolchain.comp_family() == toolchain.GCC:  # @UndefinedVariable
                # explicitely specify static lib!
                lib_names += ";libgfortran.a"
            self.cfg.update('configopts', '-D%s_LIBRARY_NAMES="%s"' % (dep, lib_names))

        # MKL
        if get_software_root('imkl') and LooseVersion(self.version) >= LooseVersion('12.12'):
            self.cfg.update('configopts', "-DTPL_ENABLE_MKL:BOOL=ON")
            self.cfg.update('configopts', '-DMKL_LIBRARY_DIRS:PATH="%s/lib/intel64"' % os.getenv('MKLROOT'))
            self.cfg.update('configopts', '-DMKL_INCLUDE_DIRS:PATH="%s/include"' % os.getenv('MKLROOT'))

        # UMFPACK is part of SuiteSparse
        suitesparse = get_software_root('SuiteSparse')
        if suitesparse:
            self.cfg.update('configopts', "-DTPL_ENABLE_UMFPACK:BOOL=ON")
            self.cfg.update('configopts', "-DTPL_ENABLE_Cholmod:BOOL=ON")
            incdir = os.path.join(suitesparse, "include")
            libdir = os.path.join(suitesparse, "lib")
            libs = ["UMFPACK", "CHOLMOD", "COLAMD", "AMD", "CCOLAMD", "CAMD"]
            libnames = [lib.lower() for lib in libs]

            # add SuiteSparse config lib, it is in recent versions of suitesparse
            libnames.append('suitesparseconfig')
            # because of "SuiteSparse_config.c:function SuiteSparse_tic: error: undefined reference to 'clock_gettime'"
            libnames.append('rt')

            # required to resolve METIS symbols in SuiteSparse's libcholmod.a
            # doesn't need to be full location, probably because it can be found via $LIBRARY_PATH
            # not easy to know whether it should come from METIS or ParMETIS...
            # see https://answers.launchpad.net/dorsal/+question/223167
            libnames.append('libmetis.a')

            self.cfg.update('configopts', '-DUMFPACK_INCLUDE_DIRS:PATH="%s"' % incdir)
            self.cfg.update('configopts', '-DUMFPACK_LIBRARY_DIRS:PATH="%s"' % libdir)
            self.cfg.update('configopts', '-DUMFPACK_LIBRARY_NAMES:STRING="%s"' % ';'.join(libnames))
            self.cfg.update('configopts', '-DCholmod_INCLUDE_DIRS:PATH="%s"' % incdir)
            self.cfg.update('configopts', '-DCholmod_LIBRARY_DIRS:PATH="%s"' % libdir)
            self.cfg.update('configopts', '-DCholmod_LIBRARY_NAMES:STRING="%s"' % ';'.join(libnames))

        # BLACS
        if get_software_root('BLACS'):
            self.cfg.update('configopts', "-DTPL_ENABLE_BLACS:BOOL=ON")
            self.cfg.update('configopts', '-DBLACS_INCLUDE_DIRS:PATH="%s"' % os.getenv('BLACS_INC_DIR'))
            self.cfg.update('configopts', '-DBLACS_LIBRARY_DIRS:PATH="%s"' % os.getenv('BLACS_LIB_DIR'))
            blacs_lib_names = os.getenv('BLACS_STATIC_LIBS').split(',')
            blacs_lib_names = [lib_re.search(x).group(1) for x in blacs_lib_names]
            self.cfg.update('configopts', '-DBLACS_LIBRARY_NAMES:STRING="%s"' % (';'.join(blacs_lib_names)))

        # ScaLAPACK
        if get_software_root('ScaLAPACK'):
            self.cfg.update('configopts', "-DTPL_ENABLE_SCALAPACK:BOOL=ON")
            self.cfg.update('configopts', '-DSCALAPACK_INCLUDE_DIRS:PATH="%s"' % os.getenv('SCALAPACK_INC_DIR'))
            self.cfg.update('configopts', '-DSCALAPACK_LIBRARY_DIRS:PATH="%s;%s"' % (os.getenv('SCALAPACK_LIB_DIR'),
                                                                                     os.getenv('BLACS_LIB_DIR')))
        # PETSc
        petsc = get_software_root('PETSc')
        if petsc:
            self.cfg.update('configopts', "-DTPL_ENABLE_PETSC:BOOL=ON")
            incdirs = [os.path.join(petsc, "include")]
            self.cfg.update('configopts', '-DPETSC_INCLUDE_DIRS:PATH="%s"' % ';'.join(incdirs))
            petsc_libdirs = [
                os.path.join(petsc, "lib"),
                os.path.join(suitesparse, "UMFPACK", "Lib"),
                os.path.join(suitesparse, "CHOLMOD", "Lib"),
                os.path.join(suitesparse, "COLAMD", "Lib"),
                os.path.join(suitesparse, "AMD", "Lib"),
                os.getenv('FFTW_LIB_DIR'),
                os.path.join(get_software_root('ParMETIS'), "Lib")
            ]
            self.cfg.update('configopts', '-DPETSC_LIBRARY_DIRS:PATH="%s"' % ';'.join(petsc_libdirs))
            petsc_libnames = ["petsc", "umfpack", "cholmod", "colamd", "amd", "parmetis", "metis"]
            petsc_libnames += [lib_re.search(x).group(1) for x in os.getenv('FFTW_STATIC_LIBS').split(',')]
            self.cfg.update('configopts', '-DPETSC_LIBRARY_NAMES:STRING="%s"' % ';'.join(petsc_libnames))

        # other Third-Party Libraries (TPLs)
        deps = self.cfg.dependencies()
        builddeps = [d['name'] for d in self.cfg.builddependencies()] + ["SuiteSparse"]
        deps = [dep['name'] for dep in deps if not dep['name'] in builddeps]
        for dep in deps:
            deproot = get_software_root(dep)
            if deproot:
                depmap = {
                    'SCOTCH': 'Scotch',
                }
                dep = depmap.get(dep, dep)
                self.cfg.update('configopts', "-DTPL_ENABLE_%s:BOOL=ON" % dep)
                incdir = os.path.join(deproot, "include")
                self.cfg.update('configopts', '-D%s_INCLUDE_DIRS:PATH="%s"' % (dep, incdir))
                libdir = os.path.join(deproot, "lib")
                self.cfg.update('configopts', '-D%s_LIBRARY_DIRS:PATH="%s"' % (dep, libdir))

        # extensions_step
        if self.cfg['all_exts']:
            self.cfg.update('configopts', "-DTrilinos_ENABLE_ALL_PACKAGES:BOOL=ON")

        else:
            for ext in self.cfg['exts_list']:
                self.cfg.update('configopts', "-DTrilinos_ENABLE_%s=ON" % ext)

        # packages to skip
        skip_exts = self.cfg['skip_exts']
        if skip_exts:
            for ext in skip_exts:
                self.cfg.update('configopts', "-DTrilinos_ENABLE_%s:BOOL=OFF" % ext)

        # building in source dir not supported
        # + if the build directory is a long path, problems like "Argument list too long" may occur
        # cfr. https://github.com/trilinos/Trilinos/issues/2434
        # so, try to create build directory with shorter path length to build in
        salt = ''.join(random.choice(ascii_letters) for _ in range(5))
        self.short_start_dir = os.path.join(build_path(), self.name + '-' + salt)
        if os.path.exists(self.short_start_dir):
            raise EasyBuildError("Short start directory %s for Trilinos already exists?!", self.short_start_dir)

        self.log.info("Length of path to original start directory: %s", len(self.start_dir))
        self.log.info("Short start directory: %s (length: %d)", self.short_start_dir, len(self.short_start_dir))

        mkdir(self.short_start_dir)
        short_src_dir = os.path.join(self.short_start_dir, 'src')
        symlink(self.start_dir, short_src_dir)
        short_build_dir = os.path.join(self.short_start_dir, 'obj')
        obj_dir = os.path.join(self.builddir, 'obj')
        mkdir(obj_dir)
        symlink(obj_dir, short_build_dir)

        # configure using cmake
        super(EB_Trilinos, self).configure_step(srcdir=short_src_dir, builddir=short_build_dir)

    def build_step(self):
        """Build with make (verbose logging enabled)."""
        super(EB_Trilinos, self).build_step(verbose=True)

    def sanity_check_step(self):
        """Custom sanity check for Trilinos."""

        # selection of libraries
        libs = ["Amesos", "Anasazi", "AztecOO", "Belos", "Epetra", "Galeri",
                "GlobiPack", "Ifpack", "Intrepid", "Isorropia", "Kokkos",
                "Komplex", "LOCA", "Mesquite", "ML", "Moertel", "MOOCHO", "NOX",
                "Pamgen", "RTOp", "Rythmos", "Sacado", "Shards", "Stratimikos",
                "Teuchos", "Tpetra", "Triutils", "Zoltan"]

        libs = [x for x in libs if x not in self.cfg['skip_exts']]

        # Teuchos was refactored in 11.2
        if LooseVersion(self.version) >= LooseVersion('11.2') and 'Teuchos' in libs:
            libs.remove('Teuchos')
            libs.extend(['teuchoscomm', 'teuchoscore', 'teuchosnumerics', 'teuchosparameterlist', 'teuchosremainder'])

        # Kokkos was refactored in 12.x, check for libkokkoscore.a rather than libkokkos.a
        if LooseVersion(self.version) >= LooseVersion('12') and 'Kokkos' in libs:
            libs.remove('Kokkos')
            libs.append('kokkoscore')

        # libgaleri was split into libgaleri-epetra & libgaleri-xpetra
        if LooseVersion(self.version) >= LooseVersion('12.6'):
            libs.remove('Galeri')
            libs.extend(['galeri-epetra', 'galeri-xpetra'])

        # Mesquite and MOOCHO packages gone in 12.18:
        if LooseVersion(self.version) >= LooseVersion('12.18'):
            libs.remove('Mesquite')
            libs.remove('MOOCHO')

        # GlobiPack package gone in 13.0:
        if LooseVersion(self.version) >= LooseVersion('13.0'):
            libs.remove('GlobiPack')

        # Get the library extension
        if self.cfg['build_shared_libs']:
            lib_ext = get_shared_lib_ext()
        else:
            lib_ext = 'a'

        custom_paths = {
            'files': [os.path.join('lib', 'lib%s.%s' % (x.lower(), lib_ext)) for x in libs],
            'dirs': ['bin', 'include']
        }

        super(EB_Trilinos, self).sanity_check_step(custom_paths=custom_paths)

    def cleanup_step(self):
        """Complete cleanup by also removing custom created short build directory."""
        remove_dir(self.short_start_dir)



##
# Copyright 2019-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Scipion, implemented as an easyblock

@author: Kenneth Hoste (Ghent University)
@author: Ake Sandgren (HPC2N, Umea University)
"""
import os

from easybuild.tools import LooseVersion
from easybuild.framework.extensioneasyblock import ExtensionEasyBlock
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.filetools import copy, mkdir, symlink
from easybuild.tools.modules import get_software_root, get_software_version
from easybuild.tools.py2vs3 import configparser
from easybuild.tools.run import run_cmd


class EB_Scipion(ExtensionEasyBlock):
    """Support for building/installing Scipion."""

    def __init__(self, *args, **kwargs):
        """Initialize Scipion-specific variables."""
        super(EB_Scipion, self).__init__(*args, **kwargs)
        # strip off 'scipion-*' part to avoid having everything in a subdirectory
        self.unpack_options = '--strip-components=1'

    def configure_step(self):
        """Custom configuration procedure for Scipion."""
        # Setup scipion.conf, protocols.conf, and host.conf
        # scipion.conf needs to be configured with paths to all the
        # dependencies, stupid but that's the way it works at the
        # moment.

        cfgfile = os.path.join(self.builddir, 'config', 'scipion.conf')

        # Create the config files and then make the subtitutions
        cmd = ' '.join([
            self.cfg['preconfigopts'],
            './scipion',
            '--config',
            cfgfile,
            'config',
            self.cfg['configopts'],
        ])
        run_cmd(cmd, log_all=True, simple=True)

        # Things that go into the BUILD section of scipion.conf
        build_params = {
            'CC': os.environ['CC'],
            'CXX': os.environ['CXX'],
            'LINKERFORPROGRAMS': os.environ['CXX'],
            'MPI_CC': os.environ.get('MPICC', 'UNKNOWN'),
            'MPI_CXX': os.environ.get('MPICXX', 'UNKNOWN'),
            'MPI_LINKERFORPROGRAMS': os.environ.get('MPICXX', 'UNKNOWN'),
        }

        package_params = {
            # Some variables we can set regardless of whether it exists as a dependency or not.
            'GCTF': 'Gctf',
            'MOTIONCOR2_BIN': 'motioncor2',
            'NVCC_INCLUDE': '',
            'NVCC_LIBDIR': '',
        }

        # EM package dependencies and the corresponding scipion.conf variable
        deps = [
            # dep name, environment variable to define, is required dep?
            ('frealign', 'FREALIGN_HOME', False),
            ('Gctf', 'GCTF_HOME', False),
            ('MotionCor2', 'MOTIONCOR2_HOME', False),
            ('RELION', 'RELION_HOME', False),
            ('VMD', 'VMD_HOME', False),
            ('Xmipp', 'XMIPP_HOME', True),
        ]

        if LooseVersion(self.version) >= LooseVersion('2.0'):
            deps.append(('EMAN2', 'EMAN2_HOME', False))
        else:
            deps.append(('EMAN2', 'EMAN2DIR', False))

        if get_software_root('ctffind'):
            if LooseVersion(get_software_version('ctffind')) >= LooseVersion('4.0'):
                deps.append(('ctffind', 'CTFFIND4_HOME', False))
            else:
                deps.append(('ctffind', 'CTFFIND_HOME', False))

        cuda_root = get_software_root('CUDA')
        build_params.update({'CUDA': bool(cuda_root)})
        if cuda_root:
            build_params.update({'CUDA_BIN': os.path.join(cuda_root, 'bin')})
            build_params.update({'CUDA_LIB': os.path.join(cuda_root, 'lib64')})

        opencv_root = get_software_root('OpenCV')
        build_params.update({'OPENCV': bool(opencv_root)})
        if opencv_root:
            build_params.update({'OPENCV_VER': get_software_version('OpenCV')})

        matlab_root = get_software_root('MATLAB')
        build_params.update({'MATLAB': bool(matlab_root)})
        if matlab_root:
            build_params.update({'MATLAB_DIR': matlab_root})

        missing_deps = []

        java_root = get_software_root('Java')
        if not java_root:
            missing_deps.append('JAVA')

        for dep, var, required in deps:
            root = get_software_root(dep)
            if root:
                package_params.update({var: root})
            elif required:
                missing_deps.append(dep)

        if missing_deps:
            raise EasyBuildError("One or more required dependencies not available: %s", ', '.join(missing_deps))

        # Here we know that Java exists
        build_params.update({'JAVA_HOME': java_root})
        jnipath = os.path.join(java_root, 'include')
        jnipath = os.pathsep.join([jnipath, os.path.join(jnipath, 'linux')])
        build_params.update({'JNI_CPPPATH': jnipath})

        cf = configparser.ConfigParser()
        cf.optionxform = str  # keep case

        try:
            cf.read(cfgfile)
        except (IOError, OSError) as err:
            raise EasyBuildError("Failed to read %s: %s", cfgfile, err)

        # Update BUILD settings
        for key in build_params:
            cf.set('BUILD', key, build_params[key])

        # Add param stuff
        for key in package_params:
            cf.set('PACKAGES', key, package_params[key])

        try:
            cf.write(open(cfgfile, 'w'))
        except (IOError, OSError) as err:
            raise EasyBuildError("Failed to write %s: %s", cfgfile, err)

    def build_step(self):
        """No build step, this is pure python code"""
        pass

    def install_step(self):
        """Custom install step for Scipion."""

        copy(['config', 'pyworkflow', 'scipion', 'software'], self.installdir)
        datadir = os.path.join(self.installdir, 'data', 'tests')
        mkdir(datadir, parents=True)
        mkdir(os.path.join(self.installdir, 'bin'))
        linktarget = os.path.join(self.installdir, 'bin', 'scipion')
        symlink(os.path.join('..', 'scipion'), linktarget, use_abspath_source=False)

    def run(self, *args, **kwargs):
        """Perform the actual Scipion package configure/installation procedure"""

        # The ExtensionEasyBlock run does unpack and patch
        super(EB_Scipion, self).run(unpack_src=True)
        self.builddir = self.ext_dir

        # configure, build, install
        self.configure_step()
        self.build_step()
        self.install_step()

    def sanity_check_step(self):
        """Custom sanity check for Scipion."""

        custom_paths = {
            'files': [os.path.join('bin', 'scipion')],
            'dirs': ['config', 'data', 'pyworkflow', 'software'],
        }
        return super(EB_Scipion, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2022-2023 Vrije Universiteit Brussel
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for Julia Packages, implemented as an easyblock

@author: Alex Domingo (Vrije Universiteit Brussel)
"""
import os
import re

from easybuild.tools import LooseVersion

import easybuild.tools.environment as env
from easybuild.framework.easyconfig import CUSTOM
from easybuild.framework.extensioneasyblock import ExtensionEasyBlock
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.modules import get_software_root, get_software_version
from easybuild.tools.filetools import copy_dir
from easybuild.tools.run import run_cmd

EXTS_FILTER_JULIA_PACKAGES = ("julia -e 'using %(ext_name)s'", "")
USER_DEPOT_PATTERN = re.compile(r"\/\.julia\/?$")


class JuliaPackage(ExtensionEasyBlock):
    """Builds and installs Julia Packages."""

    @staticmethod
    def extra_options(extra_vars=None):
        """Extra easyconfig parameters specific to JuliaPackage."""
        extra_vars = ExtensionEasyBlock.extra_options(extra_vars=extra_vars)
        extra_vars.update({
            'download_pkg_deps': [
                False, "Let Julia download and bundle all needed dependencies for this installation", CUSTOM
            ],
        })
        return extra_vars

    def set_depot_path(self):
        """
        Top directory in JULIA_DEPOT_PATH is target installation directory
        Prepend installation directory to JULIA_DEPOT_PATH
        Remove user depot from JULIA_DEPOT_PATH during installation
        see https://docs.julialang.org/en/v1/manual/environment-variables/#JULIA_DEPOT_PATH
        """
        depot_path = os.getenv('JULIA_DEPOT_PATH', [])

        if depot_path:
            depot_path = depot_path.split(os.pathsep)
        if len(depot_path) > 0:
            # strip user depot path (top entry by definition)
            if USER_DEPOT_PATTERN.search(depot_path[0]):
                self.log.debug('Temporary disabling Julia user depot: %s', depot_path[0])
                del depot_path[0]

        depot_path.insert(0, self.installdir)
        env.setvar('JULIA_DEPOT_PATH', os.pathsep.join(depot_path))

    def set_pkg_offline(self):
        """Enable offline mode of Julia Pkg"""
        if get_software_root('Julia') is None:
            raise EasyBuildError("Julia not included as dependency!")

        if not self.cfg['download_pkg_deps']:
            julia_version = get_software_version('Julia')
            if LooseVersion(julia_version) >= LooseVersion('1.5'):
                # Enable offline mode of Julia Pkg
                # https://pkgdocs.julialang.org/v1/api/#Pkg.offline
                env.setvar('JULIA_PKG_OFFLINE', 'true')
            else:
                errmsg = (
                    "Cannot set offline mode in Julia v%s (needs Julia >= 1.5). "
                    "Enable easyconfig option 'download_pkg_deps' to allow installation "
                    "with any extra downloaded dependencies."
                )
                raise EasyBuildError(errmsg, julia_version)

    def prepare_step(self, *args, **kwargs):
        """Prepare for installing Julia package."""
        super(JuliaPackage, self).prepare_step(*args, **kwargs)
        self.set_pkg_offline()
        self.set_depot_path()

    def configure_step(self):
        """No separate configuration for JuliaPackage."""
        pass

    def build_step(self):
        """No separate build procedure for JuliaPackage."""
        pass

    def test_step(self):
        """No separate (standard) test procedure for JuliaPackage."""
        pass

    def install_step(self):
        """Install Julia package with Pkg"""

        # command sequence for Julia.Pkg
        julia_pkg_cmd = ['using Pkg']
        if os.path.isdir(os.path.join(self.start_dir, '.git')):
            # sources from git repos can be installed as any remote package
            self.log.debug('Installing Julia package in normal mode (Pkg.add)')

            julia_pkg_cmd.extend([
                # install package from local path preserving existing dependencies
                'Pkg.add(url="%s"; preserve=Pkg.PRESERVE_ALL)' % self.start_dir,
            ])
        else:
            # plain sources have to be installed in develop mode
            # copy sources to install directory and install
            self.log.debug('Installing Julia package in develop mode (Pkg.develop)')

            install_pkg_path = os.path.join(self.installdir, 'packages', self.name)
            copy_dir(self.start_dir, install_pkg_path)

            julia_pkg_cmd.extend([
                'Pkg.develop(PackageSpec(path="%s"))' % install_pkg_path,
                'Pkg.build("%s")' % self.name,
            ])

        julia_pkg_cmd = ';'.join(julia_pkg_cmd)
        cmd = ' '.join([
            self.cfg['preinstallopts'],
            "julia -e '%s'" % julia_pkg_cmd,
            self.cfg['installopts'],
        ])
        (out, _) = run_cmd(cmd, log_all=True, simple=False)

        return out

    def run(self):
        """Install Julia package as an extension."""

        if not self.src:
            errmsg = "No source found for Julia package %s, required for installation. (src: %s)"
            raise EasyBuildError(errmsg, self.name, self.src)
        ExtensionEasyBlock.run(self, unpack_src=True)

        self.set_pkg_offline()
        self.set_depot_path()  # all extensions share common depot in installdir
        self.install_step()

    def sanity_check_step(self, *args, **kwargs):
        """Custom sanity check for JuliaPackage"""

        pkg_dir = os.path.join('packages', self.name)

        custom_paths = {
            'files': [],
            'dirs': [pkg_dir],
        }
        kwargs.update({'custom_paths': custom_paths})

        return ExtensionEasyBlock.sanity_check_step(self, EXTS_FILTER_JULIA_PACKAGES, *args, **kwargs)

    def make_module_extra(self):
        """
        Module has to append installation directory to JULIA_DEPOT_PATH to keep
        the user depot in the top entry. See issue easybuilders/easybuild-easyconfigs#17455
        """
        txt = super(JuliaPackage, self).make_module_extra()
        txt += self.module_generator.append_paths('JULIA_DEPOT_PATH', [''])
        return txt



##
# Copyright 2017-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing tensorflow-compresssion, implemented as an easyblock

@author: Ake Sandgren (Umea University)
"""
import glob
import os

import easybuild.tools.environment as env
from easybuild.easyblocks.generic.pythonpackage import PythonPackage
from easybuild.tools import LooseVersion
from easybuild.tools.modules import get_software_version
from easybuild.tools.run import run_cmd


class EB_tensorflow_minus_compression(PythonPackage):

    def setup_build_dirs(self):
        """Setup temporary build directories"""
        # This is either the builddir (for standalone builds) or the extension sub folder when TFC is an extension
        # Either way this folder only contains the folder with the sources and hence we can use fixed names
        # for the subfolders
        parent_dir = os.path.dirname(self.start_dir)
        # Path where Bazel will store its output, build artefacts etc.
        self.output_user_root_dir = os.path.join(parent_dir, 'bazel-root')
        # Folder where wrapper binaries can be placed, where required. TODO: Replace by --action_env cmds
        self.wrapper_dir = os.path.join(parent_dir, 'wrapper_bin')

    def configure_step(self):
        """Custom configuration procedure for TensorFlow-Compression."""

        self.setup_build_dirs()

        # Options passed to the target (build/test), e.g. --config arguments
        self.target_opts = []

    def build_step(self):
        """Custom build procedure for TensorFlow-Compression."""

        bazel_version = get_software_version('Bazel')

        # Options passed to the bazel command
        self.bazel_opts = [
            '--output_user_root=%s' % self.output_user_root_dir,
        ]

        # Environment variables and values needed for Bazel actions.
        action_env = {}
        # A value of None is interpreted as using the invoking environments value
        INHERIT = None  # For better readability

        if self.toolchain.options.get('debug', None):
            self.target_opts.append('--strip=never')
            self.target_opts.append('--compilation_mode=dbg')

        for flag in os.getenv('CFLAGS', '').split(' '):
            self.target_opts.append('--copt="%s"' % flag)

        # make Bazel print full command line + make it verbose on failures
        # https://docs.bazel.build/versions/master/user-manual.html#flag--subcommands
        # https://docs.bazel.build/versions/master/user-manual.html#flag--verbose_failures
        self.target_opts.extend(['--subcommands', '--verbose_failures'])

        self.target_opts.append('--jobs=%s' % self.cfg['parallel'])

        # include install location of Python packages in $PYTHONPATH,
        # and specify that value of $PYTHONPATH should be passed down into Bazel build environment,
        # this is required to make sure that Python packages included as extensions are found at build time,
        # see also https://github.com/tensorflow/tensorflow/issues/22395
        pythonpath = os.getenv('PYTHONPATH', '')
        env.setvar('PYTHONPATH', os.pathsep.join([os.path.join(self.installdir, self.pylibdir), pythonpath]))

        # Make TFC find our modules. LD_LIBRARY_PATH gets automatically added
        action_env['CPATH'] = INHERIT
        action_env['LIBRARY_PATH'] = INHERIT
        action_env['PYTHONPATH'] = INHERIT
        # Also export $EBPYTHONPREFIXES to handle the multi-deps python setup
        # See https://github.com/easybuilders/easybuild-easyblocks/pull/1664
        if 'EBPYTHONPREFIXES' in os.environ:
            action_env['EBPYTHONPREFIXES'] = INHERIT

        # Ignore user environment for Python
        action_env['PYTHONNOUSERSITE'] = '1'

        # Use the same configuration (i.e. environment) for compiling and using host tools
        # This means that our action_envs are (almost) always passed
        # Fully removed in Bazel 6.0 and limited effect after at least 3.7 (see --host_action_env)
        if LooseVersion(bazel_version) < LooseVersion('6.0.0'):
            self.target_opts.append('--distinct_host_configuration=false')

        for key, value in sorted(action_env.items()):
            option = key
            if value is not None:
                option += "='%s'" % value

            self.target_opts.append('--action_env=' + option)
            if bazel_version >= '3.7.0':
                # Since Bazel 3.7 action_env only applies to the "target" environment, not the "host" environment
                # As we are not cross-compiling we need both be the same -> Duplicate the setting to host_action_env
                # See https://github.com/bazelbuild/bazel/commit/a463d9095386b22c121d20957222dbb44caef7d4
                self.target_opts.append('--host_action_env=' + option)

        # Compose final command
        cmd = (
            [self.cfg['prebuildopts']]
            + ['bazel']
            + self.bazel_opts
            + ['build']
            + self.target_opts
            + [self.cfg['buildopts']]
            # specify target of the build command as last argument
            + [':build_pip_pkg']
        )

        run_cmd(' '.join(cmd), log_all=True, simple=True, log_ok=True)

        # run generated 'build_pip_pkg' script to build the .whl
        cmd = (
            'python',
            'build_pip_pkg.py',
            'bazel-bin/build_pip_pkg.runfiles/tensorflow_compression',
            self.builddir,
            self.version
        )
        run_cmd(' '.join(cmd), log_all=True, simple=True, log_ok=True)

    def install_step(self):
        """Custom install procedure for TensorFlow-Compression."""

        whl_paths = glob.glob(os.path.join(self.builddir, 'tensorflow_compression-%s-*.whl' % self.version))
        if not whl_paths:
            whl_paths = glob.glob(os.path.join(self.builddir, 'tensorflow_compression-*.whl'))
        if len(whl_paths) == 1:
            self.cfg['install_src'] = whl_paths[0]

        super(EB_tensorflow_minus_compression, self).install_step()



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing xml R, implemented as an easyblock

@author: Kenneth Hoste (Ghent University)
"""
import os

import easybuild.tools.environment as env
from easybuild.easyblocks.generic.rpackage import RPackage
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.config import build_option
from easybuild.tools.modules import get_software_root


class EB_XML(RPackage):
    """Support for installing the XML R package."""

    def install_R_package(self, *args, **kwargs):
        """Customized install procedure for XML R package, add zlib lib path to LIBS."""

        libs = os.getenv('LIBS', '')
        zlib = get_software_root('zlib')

        if zlib:
            env.setvar('LIBS', "%s -L%s" % (libs, os.path.join(zlib, 'lib')))
        elif 'zlib' in build_option('filter_deps'):
            self.log.info("zlib included in list of filtered dependencies, so no need to tweak $LIBS")
        else:
            raise EasyBuildError("zlib module not loaded (required)")

        return super(EB_XML, self).install_R_package(*args, **kwargs)
